/*
 * Copyright (c) 1998 The Santa Cruz Operation, Inc.. All Rights Reserved. 
 *                                                                         
 *        THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF THE               
 *                   SANTA CRUZ OPERATION INC.                             
 *                                                                         
 *   The copyright notice above does not evidence any actual or intended   
 *   publication of such source code.                                      
 */

#ifndef _PTHREAD_H
#define _PTHREAD_H
#ident	"@(#)sgs-head:common/head/pthread.h	1.3"

#include <sys/types.h>
#include <time.h>
#include <sched.h>

/* defines that indicate Posix.4a support
 * for current release of product
 */
#define _POSIX_REENTRANT_FUNCTIONS		1	/* IS THIS RIGHT?? */
#define _POSIX_THREAD_THREAD_MAX		0	/* IS THIS RIGHT?? */

#define PTHREAD_PRIO_NONE			1
#define PTHREAD_PRIO_INHERIT			2
#define PTHREAD_PRIO_PROTECT			3

/* reqd when _POSIX_THREAD_PROCESS_SHARED is defined
 */
#define PTHREAD_PROCESS_PRIVATE		0
#define PTHREAD_PROCESS_SHARED		1

/* reqd when _POSIX_THREAD_PRIORITY_SCHEDULING is defined
 */
#define	PTHREAD_INHERIT_SCHED		0
#define	PTHREAD_EXPLICIT_SCHED		1

#define PTHREAD_SCOPE_PROCESS		2
#define PTHREAD_SCOPE_SYSTEM		3

#define PTHREAD_MUTEX_INITIALIZER	{0,0,0,0,0,0,0,0,0,0,0,0,0,0}
#define PTHREAD_COND_INITIALIZER	{0,0,0,0,0,0,0,0}
#define PTHREAD_RWLOCK_INITIALIZER	{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,{0,0}}
#define PTHREAD_ONCE_INIT		{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}

#define	PTHREAD_CREATE_DETACHED		0
#define	PTHREAD_CREATE_JOINABLE		1

/*
 * flags for thread cancellation
 */
#define PTHREAD_CANCEL_ENABLE		0
#define PTHREAD_CANCEL_DISABLE		1
#define PTHREAD_CANCEL_DEFERRED		2
#define PTHREAD_CANCEL_ASYNCHRONOUS	4
#define PTHREAD_CANCELED		-1	/* return value if canceled */

/*
 * flags for mutex type
 */
#define PTHREAD_MUTEX_DEFAULT           0
#define PTHREAD_MUTEX_NORMAL            1
#define PTHREAD_MUTEX_ERRORCHECK        2
#define PTHREAD_MUTEX_RECURSIVE         4

#ifdef __cplusplus
extern "C" {
#endif

int pthread_atfork(void (*)(void), void (*)(void), void (*)(void));

/* Section 11.3 Mutexes
 */
int pthread_mutexattr_init(pthread_mutexattr_t *);
int pthread_mutexattr_destroy(pthread_mutexattr_t *);
int pthread_mutexattr_getpshared(const pthread_mutexattr_t *, int *);
int pthread_mutexattr_setpshared(pthread_mutexattr_t *, int);

int pthread_mutex_init(pthread_mutex_t *, const pthread_mutexattr_t *);
int pthread_mutex_destroy(pthread_mutex_t *);

int pthread_mutex_lock(pthread_mutex_t *);
int pthread_mutex_trylock(pthread_mutex_t *);
int pthread_mutex_unlock(pthread_mutex_t *);

/* Section 11.4 Condition variables
 */
int pthread_condattr_init(pthread_condattr_t *);
int pthread_condattr_destroy(pthread_condattr_t *);
int pthread_condattr_getpshared(const pthread_condattr_t *, int *);
int pthread_condattr_setpshared(pthread_condattr_t *, int);

int pthread_cond_init(pthread_cond_t *, const pthread_condattr_t *);
int pthread_cond_destroy(pthread_cond_t *);

int pthread_cond_signal(pthread_cond_t *);
int pthread_cond_broadcast(pthread_cond_t *);

int pthread_cond_wait(pthread_cond_t *, pthread_mutex_t *);
int pthread_cond_timedwait(pthread_cond_t *, pthread_mutex_t *,
	const struct timespec *);

/* Section 13.5 Thread scheduling
 * defined  under _POSIX_THREAD_PRIORITY_SCHEDULING
 */
int pthread_attr_setscope(pthread_attr_t *, int);
int pthread_attr_getscope(const pthread_attr_t *, int *);

int pthread_attr_setinheritsched(pthread_attr_t *, int);
int pthread_attr_getinheritsched(const pthread_attr_t *, int *);

int pthread_attr_setschedpolicy(pthread_attr_t *, int);
int pthread_attr_getschedpolicy(const pthread_attr_t *, int *);

int pthread_attr_setschedparam(pthread_attr_t *,
	const struct sched_param *);
int pthread_attr_getschedparam(const pthread_attr_t *,
	struct sched_param *);

int pthread_getschedparam(pthread_t, int *, struct sched_param *);
int pthread_setschedparam(pthread_t, int, const struct sched_param *);

/* end _POSIX_THREAD_PRIORITY SCHEDULING
 */

/* Section 16 Thread management
 */
int pthread_attr_init(pthread_attr_t *);
int pthread_attr_destroy(pthread_attr_t *);
int pthread_attr_setstacksize(pthread_attr_t *, size_t);
int pthread_attr_getstacksize(const pthread_attr_t *, size_t *);
int pthread_attr_setstackaddr(pthread_attr_t *, void *);
int pthread_attr_getstackaddr(const pthread_attr_t *, void **);
int pthread_attr_setdetachstate(pthread_attr_t *, int);
int pthread_attr_getdetachstate(const pthread_attr_t *, int *);
int pthread_create(pthread_t *, const pthread_attr_t *,
	void * (*)(void *), void *);
int pthread_join(pthread_t, void **);
int pthread_detach(pthread_t);
void pthread_exit(void *);
pthread_t pthread_self(void);
int pthread_equal(pthread_t, pthread_t);
int pthread_once(pthread_once_t *, void (*)(void));


/* Section 17 Thread-specific data
 */
int pthread_key_create(pthread_key_t *, void (*)(void *));
int pthread_key_delete(pthread_key_t);
int pthread_setspecific(pthread_key_t, const void *);
void *pthread_getspecific(pthread_key_t);


/*
 * Section 18 Thread cancellation
 */
int pthread_cancel(pthread_t);
void pthread_cleanup_push(void (*)(void *), void *);
void pthread_cleanup_pop(int);
int pthread_cancel(pthread_t);
int pthread_setcancelstate(int, int *);
int pthread_setcanceltype(int, int *);
void pthread_testcancel(void);

/*
 * X/Open Additions
 */
int pthread_getconcurrency(void);
int pthread_setconcurrency(int);
int pthread_rwlockattr_init(pthread_rwlockattr_t *);
int pthread_rwlockattr_destroy(pthread_rwlockattr_t *);
int pthread_rwlockattr_getpshared(const pthread_rwlockattr_t *, int *);
int pthread_rwlockattr_setpshared(pthread_rwlockattr_t *, int);
int pthread_rwlock_init(pthread_rwlock_t *, const pthread_rwlockattr_t *);
int pthread_rwlock_destroy(pthread_rwlock_t *);
int pthread_rwlock_rdlock(pthread_rwlock_t *);
int pthread_rwlock_tryrdlock(pthread_rwlock_t *);
int pthread_rwlock_wrlock(pthread_rwlock_t *);
int pthread_rwlock_trywrlock(pthread_rwlock_t *);
int pthread_rwlock_unlock(pthread_rwlock_t *);

#ifdef __cplusplus
}
#endif

#endif /*_PTHREAD_H*/
